@@ -1,4 +1,5 @@
 import dis
+import sys
 from functools import lru_cache
 from types import CodeType
 from typing import FrozenSet
@@ -7,6 +8,7 @@ from scalene.scalene_statistics import ByteCodeIndex
 
 
 class ScaleneFuncUtils:
+    """Utility class to determine whether a bytecode corresponds to function calls."""
 
     # We use these in is_call_function to determine whether a
     # particular bytecode is a function call.  We use this to
@@ -17,6 +19,7 @@ class ScaleneFuncUtils:
             dis.opmap[op_name]
             for op_name in dis.opmap
             if op_name.startswith("CALL_FUNCTION")
+            or (sys.version_info >= (3, 11) and op_name.startswith("CALL"))
         }
     )
 
