@@ -1,27 +1,30 @@
 import queue
 import threading
+from typing import Any, Generic, Optional, TypeVar
+
+T = TypeVar("T")
 
 
-class ScaleneSigQueue:
-    def __init__(self, process):
-        self.queue = queue.SimpleQueue()
+class ScaleneSigQueue(Generic[T]):
+    def __init__(self, process: Any) -> None:
+        self.queue: queue.SimpleQueue[Optional[T]] = queue.SimpleQueue()
         self.process = process
-        self.thread = None
-        self.lock = threading.RLock() # held while processing an item
+        self.thread: Optional[threading.Thread] = None
+        self.lock = threading.RLock()  # held while processing an item
 
-    def put(self, item):
+    def put(self, item: Optional[T]) -> None:
         self.queue.put(item)
 
-    def get(self):
+    def get(self) -> Optional[T]:
         return self.queue.get()
 
-    def start(self):
+    def start(self) -> None:
         # We use a daemon thread to defensively avoid hanging if we never join with it
         if not self.thread:
             self.thread = threading.Thread(target=self.run, daemon=True)
             self.thread.start()
 
-    def stop(self):
+    def stop(self) -> None:
         if self.thread:
             self.queue.put(None)
             # We need to join all threads before a fork() to avoid an inconsistent
@@ -29,10 +32,10 @@ class ScaleneSigQueue:
             self.thread.join()
             self.thread = None
 
-    def run(self):
+    def run(self) -> None:
         while True:
             item = self.queue.get()
-            if item == None:  # None == stop request
+            if item is None:  # None => stop request
                 break
             with self.lock:
                 self.process(*item)
